import java.util.*;

class Human extends GameCharacter {
  //characteristics of the human
  boolean reactsToNoiseItem;
  float fieldOfView;                             //angle in degrees, how broad is the humans field of view
  int sightDistance;                             //how far can the human see
  boolean hasLamp;

  //runtime characteristics
  int posX, posY;
  int nextX, nextY;
  int state;
  WorldObject goal;                                                 //current goal
  int [][] goals = new int [3][2];                                  //3 randomly generated points where the npc wanders inbetween. 
  PVector orientation = new PVector(1, 0);                          //where is the NPC looking, initial state does not matter.
  int targetCharacter;
  boolean [][] visible = new boolean [NUM_TILES_X][NUM_TILES_Y];    //tiles the human can see. In a world sized array, since reallocating space would probably take more time than working like this.
  WorldObject noiseItem;                                            //target that can be triggered, affects reactToNoiseItem = true - humans

  //for pathfinding
  HashSet<WorldObject> visited = new HashSet<WorldObject>();                                //tracks visited tiles      
  Queue<WorldObject> adjacent = new LinkedList<WorldObject>();                              //stores the tiles that need checking
  HashMap <WorldObject, WorldObject> previous = new HashMap<WorldObject, WorldObject>();    //for backtracking. stores the predecessors of the visited tiles

  //audio
  AudioPlayer giveUp;
  AudioPlayer heardSomething;
  AudioPlayer talking;
  AudioPlayer greeting;
  int soundNumber = round(random(-0.49, 2.5));

  //timer & cooldowns
  final int PATROL_RADIUS = 8;

  final int TIME_LOOKING_AROUND = 5000;

  final int ALIEN_DETECTION_RADIUS = 4; 
  final int HUMAN_DETECTION_RADIUS = 6;

  float lastAlerted = 0.0;
  final float ALERT_COOLDOWN = 10000.0; 
  final float ALERT_DELAY = 1000;

  float lastTalk = 0;
  final float TALKING_COOLDOWN = 10000;
  final float MAX_TALK_DURATION = 5000;

  float lastNoiseAlert = 0.0;
  final float NOISE_COOLDOWN = 1000;

  float stateStartTime = 0;

  final float ENDURANCE = 6000;   
  float chasingTimer = 0;

  //states
  final int STILL = 0;
  final int PATROLING = 1;
  final int CHASING = 2;
  final int GREETING = 3;
  final int TALKING = 4;
  final int LOOKING = 5;
  final int HEARD_NOISE_ITEM = 6;

  Human(int _posX, int _posY, int _state, float _fieldOfView, int _sightdistance, boolean _hasLamp, boolean _reactsToNoiseItem) {
    characterID = super.characterID;
    posX = _posX;
    posY = _posY;
    state = _state;
    reactsToNoiseItem = _reactsToNoiseItem;
    sightDistance = _sightdistance;
    fieldOfView = _fieldOfView;
    hasLamp = _hasLamp;

    switch (soundNumber) {
    case 0:
      giveUp = minim.loadFile("sound/humans/zapsplat_human_male_sigh_tired_19877.mp3");
      heardSomething = minim.loadFile("sound/humans/zapsplat_human_male_grunt_confused_002_19853.mp3");
      talking = minim.loadFile("sound/humans/Humans_talkingXD.mp3");
      greeting = minim.loadFile("sound/humans/happyHumanFindingHumans.mp3");
      break;
    case 1:
      giveUp = minim.loadFile("sound/humans/zapsplat_human_male_sigh_tired_19877_2.mp3");
      heardSomething = minim.loadFile("sound/humans/zapsplat_human_male_grunt_confused_002_19853_2.mp3");
      talking = minim.loadFile("sound/humans/Humans_talkingXD_2.mp3");
      greeting = minim.loadFile("sound/humans/happyHumanFindingHumans_2.mp3");
      break;
    case 2:
      giveUp = minim.loadFile("sound/humans/zapsplat_human_male_sigh_tired_19877_3.mp3");
      heardSomething = minim.loadFile("sound/humans/zapsplat_human_male_grunt_confused_002_19853_3.mp3");
      talking = minim.loadFile("sound/humans/Humans_talkingXD_3.mp3");
      greeting = minim.loadFile("sound/humans/happyHumanFindingHumans_3.mp3");
      break;
    }

    generateGoals();

    if (state != STILL) {
      goal = getNewGoal(posX, posY);
      state = PATROLING;
    }
  }

  void move() {
    redrawBG(posX, posY);      //for smoother drawing
    if (state == CHASING) {
      redrawBG(posX, posY-1);
    }

    switch(state) {
    case STILL:  //do nothing
      break;

      //move towards the next goal. if you're there, change state to Looking
    case PATROLING: 
      setNextStep(posX, posY, goal.x, goal.y);
      orientation.set(nextX-posX, nextY-posY);
      orientation.normalize();
      goToNext();
      if (goal == wObjectsInLevel[posX][posY]) {
        state = LOOKING;
        stateStartTime = millis();
      }
      break;

      //chasing aliens. after a delay occured, move towards the targeted alien. After some time give up and return to Patroling
    case CHASING:
      if ((millis()-lastAlerted)>ALERT_DELAY) {
        if (chasingTimer < ENDURANCE) {
          if (targetCharacter == alien1.characterID) {
            setNextStep(posX, posY, alien1.posX, alien1.posY);
            orientation.set(alien1.posX-posX, alien1.posY - posY);
          } else {
            setNextStep(posX, posY, alien2.posX, alien2.posY);
            orientation.set(alien2.posX-posX, alien2.posY - posY);
          }
          orientation.normalize();
          goToNext();
          chasingTimer = (millis() - stateStartTime);
        } else {
          chasingTimer = 0;
          this.giveUp.rewind();
          this.giveUp.play();
          goal = getNewGoal(-1, -1);
          state = PATROLING;
          stateStartTime = millis();
        }
      }
      break;

      //after spotting a human, greet him and move closer. Once there, start talking
    case GREETING: 
      for (Human human : humans) {
        println(targetCharacter);
        if (targetCharacter == human.characterID) {
          setNextStep(posX, posY, human.posX, human.posY); 
          orientation.set(human.posX-posX, human.posY - posY);
          orientation.normalize();
          if (dist(posX, posY, human.posX, human.posY)>1) {
            goToNext();
          } else {
            state = TALKING;
            this.talking.rewind();
            this.talking.play();
            human.state = TALKING;
            human.talking.rewind();
            human.talking.play();
            stateStartTime = millis();
            human.stateStartTime = millis();
          }
        }
      }
      break;

      //talk for MAX_TALK_DURATION seconds, then, start patroling again
    case TALKING:
      float talkDuration = (millis()-stateStartTime);           
      if (MAX_TALK_DURATION - talkDuration <= 0) {
        lastTalk = millis();
        talkDuration = 0;
        goal = getNewGoal(posX, posY);
        state = PATROLING;
        stateStartTime = millis();
      }
      break;

      //After reaching a goal, look around for a few seconds. Then, start patroling again
    case LOOKING:
      float fractionOfTime = (millis()-stateStartTime)/(TIME_LOOKING_AROUND/2);
      if (fractionOfTime <= 0.5) {
        orientation.rotate(PI*fractionOfTime);
      } else if (fractionOfTime >= 0.5 && fractionOfTime <= 1) {
        orientation.rotate(PI*fractionOfTime);
      } else {
        goal = getNewGoal(goal.x, goal.y);  
        state = PATROLING;
        stateStartTime = millis();
      }
      break;

      //After NoiseItem was triggered, move towards it. When there interact with it and Look around.
    case HEARD_NOISE_ITEM:
      setNextStep(posX, posY, goal.x, goal.y);
      if (posX == goal.x && posY == goal.y) {
        ((NoiseItem)wObjectsInLevel[noiseItem.x][noiseItem.y]).act();
        state = LOOKING;
        stateStartTime = millis();
      }
      orientation.set(goal.x-posX, goal.y - posY);
      orientation.normalize();
      goToNext();
      break;

    default: //do nothing
      break;
    }
  }


  void draw() {
    image(humanImg, posX, posY, 1, 1); 
    if (state == TALKING) {
      image(speechBubbleImg, posX, posY-1, 1, 1);
    } else if (state == CHASING) {
      image(exclamationPointImg, posX, posY-1, 1, 1);
    }
    fov(posX, posY);    //determine visible tiles
    lightTiles();       
    detectAliens();    
    checkSurroundings();
  }


  //lights visible World tiles if the npc has a lamp
  void lightTiles() {
    if (hasLamp) {
      for (int i= 0; i<NUM_TILES_X; i++) {                              
        for (int o = 0; o<NUM_TILES_Y; o++) {
          if (!litTiles[i][o]) {
            litTiles[i][o] = visible[i][o];
          }
        }
      }
    }
  }

  /*
  fills the visible array with tiles that are visible to the NPC. They are based on the fieldOfView, orientation and sightDistance of the npc and the surrounding geometry
   creator: Mithreindeir on Youtube, "How To - Field of View Algorithm For roguelikes", abgerufen am 22.12.2020, übersetzt in Java aus C (vermutlich)
   */
  void fov(int posX, int posY) {          
    float x;
    float y;
    final float DEG_RAD_CONV = PI/180;
    int drehung = round((orientation.heading()/DEG_RAD_CONV)-(fieldOfView/2));  //orientation of the human converted in degrees

    for (int i= 0; i<NUM_TILES_X; i++) {                              //reset visible Array
      for (int o = 0; o<NUM_TILES_Y; o++) {
        visible[i][o] = false;
      }
    }


    for (int i = 0 + drehung; i<fieldOfView + drehung; i+=4) {                //turn step by step
      float deg = i*DEG_RAD_CONV;

      x = (cos(deg) * sightDistance) + posX;                                  //set lines to endpoints
      y = (sin(deg) * sightDistance) + posY;

      int d = round(dist(posX, posY, x, y));

      for (int u = 0; u<d; u++) {
        int tileX = round(lerp(posX, x, u/(float)d));                         //interpolate along the lines
        int tileY = round(lerp(posY, y, u/(float)d));        

        if (tileX < 0 || tileX >= NUM_TILES_X) {                              //ignore out of bounds
          continue;
        }    //check bounds
        if (tileY < 0 || tileY >= NUM_TILES_Y) {
          continue;
        }

        if (!wObjectsInLevel[tileX][tileY].walkable) {                        //if you hit a wall, door or other dont continue
          break;
        }
        visible[tileX][tileY] = true;                                         //set the current (interpolated) tile as visible
      }
    }
  }

  /*
  if aliens are in view and lit or on the same tile as the npc, end game
   */
  void detectAliens() {
    if (visible[alien1.posX][alien1.posY]) {
      if (litTiles[alien1.posX][alien1.posY] || (alien1.posX == posX && alien1.posY == posY)) {
        MainGame.currentGameMode = MainGame.glm; 
        MainGame.currentGameMode.start(false);
      }
    }
    if (visible[alien2.posX][alien2.posY]) {
      if (litTiles[alien2.posX][alien2.posY] || (alien2.posX == posX && alien2.posY == posY)) {
        MainGame.currentGameMode = MainGame.glm; 
        MainGame.currentGameMode.start(false);
      }
    }
  }

  /*
  if not distracted, "listen" for aliens/humans in range, react accordingly
   */
  void checkSurroundings() { 
    if (state != CHASING && state != TALKING && state != HEARD_NOISE_ITEM) {                                                                             //if not heavily distracted (chasing a alien, talking or looking for the noise)
      if (dist(posX, posY, alien1.posX, alien1.posY)<= ALIEN_DETECTION_RADIUS || dist(posX, posY, alien2.posX, alien2.posY)<= ALIEN_DETECTION_RADIUS) {  //is any alien in detection distance
        if (dist(posX, posY, alien1.posX, alien1.posY)<= ALIEN_DETECTION_RADIUS && (millis()-lastAlerted >= ALERT_COOLDOWN)) {                           //is it alien 1 and is the alert off cooldown?
          targetCharacter = alien1.characterID;
          lastAlerted = millis();
          state = CHASING;
          this.heardSomething.rewind();
          this.heardSomething.play();
          stateStartTime = millis();
        }
        if (dist(posX, posY, alien2.posX, alien2.posY)<dist(posX, posY, alien1.posX, alien1.posY) && (millis()-lastAlerted >= ALERT_COOLDOWN)) {        //is alien 2 closer and the alert off cooldown?
          targetCharacter = alien2.characterID;
          lastAlerted = millis();
          state = CHASING;
          this.heardSomething.rewind();
          this.heardSomething.play();
          stateStartTime = millis();
        }
      } else {                                                                                                                                          //aliens have priority over humans
        if (state != GREETING) {
          for (Human human : humans) {                                                                                                                  //check all humans
            if (this == human) {                                                                                                                        //ignore yourself
              continue;
            }
            if (dist(posX, posY, human.posX, human.posY)<= HUMAN_DETECTION_RADIUS && (millis()-lastTalk >= TALKING_COOLDOWN)) {                        //if close, start the greeting :)
              targetCharacter = human.characterID;
              human.targetCharacter = this.characterID;
              state = GREETING;
              this.greeting.rewind();
              this.greeting.play();
              human.state = GREETING;
              human.greeting.rewind();
              human.greeting.play();
              stateStartTime = millis();
              human.stateStartTime = millis();
            }
          }
        }
      }
    }
  }


  /*
  generates 3 random goals in PATROL_RADIUS range
   */
  void generateGoals() {
    randomSeed(millis());
    int randomX = 0;
    int randomY = 0;
    for (int i = 0; i<=2; i++) {                                                   //change here if one wants to use more random goals.
      do {                                                                         //fill with random points in the patrol_radius
        randomX = posX;                                                            //if initialized with STILL, we only want them to move if called. Else, they should walk back to start -> all entries are posX/posY
        randomY = posY;
        if (state == STILL) {                                                      //in 1 case choose a different one, so getNewGoal doesnt loop
          if (i%2 == 1) {
            randomX = posX-1;
            randomY = posY;
          }
        } else {
          randomY = posY + round(random(-PATROL_RADIUS, PATROL_RADIUS));
          randomX = posX + round(random(-PATROL_RADIUS, PATROL_RADIUS));
        }
      } while (!isWalkableForHumans(randomX, randomY));
      goals[i][0] = randomX;
      goals[i][1] = randomY;
    }
  }

  /*
  returns true if the tile at (x/y) is walkable for humans
   */
  boolean isWalkableForHumans(int x, int y) {              
    if (x<0 || x>=NUM_TILES_X|| y<0 || y>=NUM_TILES_Y) {                                                                                    //if out of bounds
      return false;
    } 

    if (wObjectsInLevel[x][y].walkable || (wObjectsInLevel[x][y] instanceof DoorPiece && ((DoorPiece)wObjectsInLevel[x][y]).humanCanOpen))  //if walkable or a openable door
    {
      if ((wObjectsInLevel[x][y] instanceof Ground) && ((Ground)wObjectsInLevel[x][y]).onlyAliens) {                                        //if its a path only for aliens -functionality for a planned level-
        return false;
      } else {
        return true;
      }
    } else {
      return false;
    }
  }

  /*
  determines the next Step on the way to the goal. Done with Breadth-First-Search
   */
  void setNextStep(int currentX, int currentY, int destinationX, int destinationY) {

    if (currentX == destinationX && currentX == destinationY) {                              //trivial, no path needed
      state = LOOKING;
      nextX = currentX;
      nextY = currentY;
      return;
    }

    WorldObject current;
    WorldObject destination;
    WorldObject temp;

    current = wObjectsInLevel[currentX][currentY]; 
    destination = wObjectsInLevel[destinationX] [destinationY];

    adjacent.add(current);

    while (!adjacent.isEmpty()) {                                                                                                  //as long as there are tiles that need to be checked
      current = adjacent.remove();

      if (current == destination) {                       //we are at the destination, so add last step and start backtracking.
        WorldObject next;
        next = backtrack(previous, destination);          //set the next tile on the path to the goal
        nextX = next.x;
        nextY = next.y;
        visited.clear();                                  //reset
        adjacent.clear();
        previous.clear();
        return;
      }

      //check north
      if ( isWalkableForHumans(current.x, current.y-1)) {
        temp = wObjectsInLevel[current.x][current.y-1];
        if (!(visited.contains(temp)) && isWalkableForHumans(current.x, current.y-1)) {                                               //if its not already visited and walkable
          adjacent.add(temp);                                                                                                       //add to the list of tiles to be checked
          previous.put(temp, current);                                                                                              //safe the predecessor in the designated table
        }
      }

      //check east
      if ( isWalkableForHumans(current.x+1, current.y)) {                                                                             //see above
        temp = wObjectsInLevel[current.x+1][current.y];
        if (!(visited.contains(temp)) && isWalkableForHumans(temp.x, temp.y)) {
          adjacent.add(temp);
          previous.put(temp, current);
        }
      }

      //check south
      if ( isWalkableForHumans(current.x, current.y+1)) {                                                                             //see above
        temp = wObjectsInLevel[current.x][current.y+1];
        if (!(visited.contains(temp)) && isWalkableForHumans(temp.x, temp.y)) {
          adjacent.add(temp);
          previous.put(temp, current);
        }
      }

      //check west
      if ( isWalkableForHumans(current.x-1, current.y)) {                                                                             //see above
        temp = wObjectsInLevel[current.x-1][current.y];
        if (!visited.contains(temp)) {
          adjacent.add(temp);
          previous.put(temp, current);
        }
      }
      visited.add(current);                                                                                                         //add current tile to the visited list
    }

    goal = getNewGoal(destination.x, destination.y);                                                                                //unreachable goalPosition unreachable from current Position, set a new goalPosition.
    nextX = current.x;
    nextY = current.y;
  }

  /*
  goes along the predecessors from the destination to the start
   */
  WorldObject backtrack(HashMap <WorldObject, WorldObject> previous, WorldObject destination) {
    WorldObject temp;
    temp = destination;
    try {
      while (!(previous.get(temp).x == posX && previous.get(temp).y == posY)) {    //keep finding the previous tile until arriving at the tile next to the starting tile.
        temp = previous.get(temp);
      }
      return temp;
    } 
    catch (Exception e) {
      return temp;
    }
  } 

  /*
  move to the next tile, if there is a door in the way open it
   */
  void goToNext() {
    if (wObjectsInLevel[nextX][nextY] instanceof DoorPiece) {
      ((DoorPiece)wObjectsInLevel[nextX][nextY]).act();
    }
    posX = nextX;
    posY = nextY;
  }

  /*
  gets called by the noiseItem, triggers HEARD_NOISE_ITEM
   */
  void reactToNoise(WorldObject noise, WorldObject neighbour) {
    if (reactsToNoiseItem && millis()-lastNoiseAlert >= NOISE_COOLDOWN) {
      lastNoiseAlert = millis();
      state = HEARD_NOISE_ITEM;
      goal = neighbour;
      noiseItem = noise;
      stateStartTime  = millis();
    }
  }

  /*
  chooses one of the goals in goals[][] as new destination
   */
  WorldObject getNewGoal(int oldGoalX, int oldGoalY) {
    byte random;
    int x;
    int y;
    do {
      random  = (byte)round(random(-0.499, 2.499));
    } while (goals[random][0] == oldGoalX && goals[random][1] == oldGoalY);
    x = goals[random][0];                              
    y = goals[random][1];
    return wObjectsInLevel[x][y];
  }
}
