class Alien extends GameCharacter {

  char whichAlien;                          //Which alien is it? 1 or 2
  boolean canPressButton;

  int posX;                                 //position
  int posY;
  boolean inGoal;                           //after all ufo pieces are found, aliens need to be in the goal Tile


  Alien(int _posX, int _posY, char _whichAlien) {
    characterID = super.characterID;

    whichAlien = _whichAlien;
    if (whichAlien == '1') {
      canPressButton = true;
    } else {
      canPressButton = false;
    }

    posX = _posX;
    posY = _posY;

    inGoal = false;
  }

  public void draw() {
    wObjectsInLevel[posX][posY].draw();      //draw object underneath the alien
    if (whichAlien == '1') {                 //then the alien
      image(alien1Img, posX, posY, 1, 1);
    } else {
      image(alien2Img, posX, posY, 1, 1);
    }
  }

  /*
  GameRunningMode calls recieveInput, decides the action dependend on the position & alien
   */
  public void recieveInput(int _x, int _y) {
    int nextX = posX + _x;
    int nextY = posY + _y;

    if ((nextX < NUM_TILES_X) && (nextX >= 0) && (nextY < NUM_TILES_Y) && (nextY >= 0)) {                                                  // in bounds?

      if (wObjectsInLevel[nextX][nextY] instanceof InteractiveWorldObject && !(wObjectsInLevel[nextX][nextY] instanceof DoorPiece)) {      // interact with InteractiveWorldObjects that aren't just for humans (= doors)
        InteractiveWorldObject object = (InteractiveWorldObject)wObjectsInLevel[nextX][nextY];

        if (!(object instanceof Button)) {                                                                                                 // interact with objects directly (not with buttons)
          if (object instanceof Box) {
            if (this.whichAlien == '2') {
              Box b = (Box) object;
              b.tryMoving(_x, _y);
            }
          } else {
            object.act();
          }
        }
      }

      if (wObjectsInLevel[nextX][nextY].walkable) {                                                                                       // check if walkable (= non-interactive objects or buttons)
        if (wObjectsInLevel[nextX][nextY] instanceof Ground) {                                                                            // move if ground is walkable for aliens
          Ground g = (Ground) wObjectsInLevel[nextX][nextY];

          if (g.onlyHumans) {                                        //funtionality for a planned level
            // don't move
          } else {
            interactWithButton(nextX, nextY);                                                                                    // button acts when alien 2 walks off
            move(nextX, nextY);
          }
        } else {                                                                                                                          // button acts when alien 2 walks on
          interactWithButton(nextX, nextY);         
          move(nextX, nextY);
        }
      }
      if (wObjectsInLevel[nextX][nextY] instanceof Wall) {                                                                                // move if alien 1 is trying to go through a wall with hole
        Wall w = (Wall) wObjectsInLevel[nextX][nextY];
        if (w.hasSmallHole && this.whichAlien == '1') {
          move(nextX, nextY);
        }
      }
    }
  }

  /*
  moves the alien to the next Tile
   */
  public void move(int nextX, int nextY) {
    redrawBG(posX, posY);
    posX = nextX;
    posY = nextY;

    if (grm.allUfoPiecesCollected) {                                                             // when all UfoPieces are collected, check if the alien is in the goal
      if (posX == goalX1 || posX == goalX2) {
        if (posY == goalY1 || posY == goalY2) {
          inGoal = true;
        } else {
          inGoal = false;
        }
      } else {
        inGoal = false;
      }
    }
  }

  /* 
   checks if alien 2 walked on or off a button and acts accordingly 
   */
  public void interactWithButton(int nextX, int nextY) {
    if (this.whichAlien == '2') {
      if (wObjectsInLevel[posX][posY] instanceof Button) {
        Button b = (Button) wObjectsInLevel[posX][posY];
        b.act();
      } else if (wObjectsInLevel[nextX][nextY] instanceof Button) {
        Button b = (Button) wObjectsInLevel[nextX][nextY];
        b.act();
      }
    }
  }
}
