java.util.LinkedList<DoorPiece> doorPieces = new java.util.LinkedList();

// builds door out of correct DoorPieces
class DoorMaker extends WorldObject {
  int doorID;
  String whichGround;

  DoorMaker(int _x, int _y, int _doorID, String _whichObject, String _whichGround) {
    x = _x;
    y = _y;
    whichObject = _whichObject;
    doorID = _doorID;
    whichGround = _whichGround;
  }
  //---------------------------------------------------------

  void buildDoor() {
    switch (whichObject) {
    case "garden gate":
      wObjectsInLevel[x][y] = new DoorPiece(x, y, doorID, "garden gate post", whichGround);
      wObjectsInLevel[x+3][y] = new DoorPiece(x+3, y, doorID, "garden gate post", whichGround);
      wObjectsInLevel[x][y-3] = new DoorPiece(x, y-3, doorID, "garden gate post", whichGround);
      wObjectsInLevel[x+1][y] = new DoorPiece(x+1, y, doorID, "garden gate left", whichGround);
      wObjectsInLevel[x+2][y] = new DoorPiece(x+2, y, doorID, "garden gate right", whichGround);
      break;
    case "barrier":
      wObjectsInLevel[x][y] = new DoorPiece(x, y, doorID, "barrier left", whichGround);
      for (int i = 1; i <= 4; i++) {
        wObjectsInLevel[x+i][y] = new DoorPiece(x+i, y, doorID, "barrier middle", whichGround);
      }
      wObjectsInLevel[x+5][y] = new DoorPiece(x+5, y, doorID, "barrier right", whichGround);
      break;
    case "door":
      wObjectsInLevel[x][y] = new DoorPiece(x, y, doorID, "door corner", whichGround);
      wObjectsInLevel[x][y-1] = new DoorPiece(x, y-1, doorID, "door bottom", whichGround);
      wObjectsInLevel[x][y-2] = new DoorPiece(x, y-2, doorID, "door top", whichGround);      
      break;
    }
  }
}
//---------------------------------------------------------


// door has to be made out of several object, each with their own x and y coordinates for humans to be able to walk through them
class DoorPiece extends InteractiveWorldObject {
  boolean originalState;                  // door can have 2 states
  String whichGround;                     // which type of ground would theoretically lie unnder the door
  boolean humanCanOpen;
  int doorID;                             // which door the piece belongs to
  boolean acted;
  int timeLastActed = timeOutTime;        // door can only act once every second 

  DoorPiece(int _x, int _y, int _doorID, String _whichObject, String _whichGround) {
    doorPieces.add(this);
    x = _x;
    y = _y;
    walkable = false;
    originalState = true;
    whichObject = _whichObject;
    whichGround = _whichGround;
    doorID = _doorID;
    acted = false;

    switch (whichObject) {
      // garden gate
    case "garden gate post":
      humanCanOpen = false;
      break;
    case "garden gate left":
      humanCanOpen = true;
      break;
    case "garden gate right":
      humanCanOpen = true;
      break;
      // barrier
    case "barrier left":
      humanCanOpen = false;
      break;
    case "barrier middle":
      humanCanOpen = false;
      break;
    case "barrier right":
      humanCanOpen = false;
      break;
      // door
    case "door corner":
      humanCanOpen = false;
      break;
    case "door top":
      humanCanOpen = true;
      break;
    case "door bottom":
      humanCanOpen = true;
      break;
    }
  }
  //---------------------------------------------------------

  void draw() {
    switch (whichObject) {
      // garden gate
    case "garden gate post":
      fill(#a99173);
      ellipseMode(CORNER);
      circle(x, y, 1);
      break;
    case "garden gate left":
      image(gardenGateLeftImg, x, y, 1, 1);
      break;
    case "garden gate right":
      image(gardenGateRightImg, x, y, 1, 1);
      break;
    case "garden gate top":
      image(gardenGateTopImg, x, y, 1, 1);
      break;
    case "garden gate bottom":
      image(gardenGateBottomImg, x, y, 1, 1);
      break;
      // barrier
    case "barrier left":
      image(barrierLeftImg, x, y, 1, 1);
      break;
    case "barrier middle":
      image(barrierMiddleImg, x, y, 1, 1);
      break;
    case "barrier right":
      image(barrierRightImg, x, y, 1, 1);
      break;
      // door
    case "door corner":
      fill(31, 30, 28);
      ellipseMode(CORNER);
      circle(x, y, 1);
      rectMode(CORNER);
      rect(x, y, 0.5, 0.5);
      rect(x, y+0.5, 1, 0.5);
      break;
    case "door top":
      image(doorTopImg, x, y, 1, 1);
      break;
    case "door bottom":
      image(doorBottomImg, x, y, 1, 1);
      break;
    case "door left":
      image(doorLeftImg, x, y, 1, 1);
      break;
    case "door right":
      image(doorRightImg, x, y, 1, 1);
      break;
    }

    if (timeLastActed > timer) {
      acted = false;
    }
  }
  //---------------------------------------------------------

  void act() {
    if (!acted) {
      // from original state to other
      if (originalState) {
        originalState = false;
        fill(grm.world.bg);                // color for redrawing the background on changing tiles
        rectMode(CORNER);
        switch (whichObject) {
          // garden gate
        case "garden gate left":
          wObjectsInLevel[x][y] = new Ground(x, y, whichGround);
          rect(x, y, 1, 1);
          x = x-1;
          y = y-1;
          whichObject = "garden gate bottom";
          wObjectsInLevel[x][y] = this;
          break;
        case "garden gate right":
          wObjectsInLevel[x][y] = new Ground(x, y, whichGround);
          rect(x, y, 1, 1);
          x = x-2;
          y = y-2;
          whichObject = "garden gate top";
          wObjectsInLevel[x][y] = this;
          break;
          // barrier
        case "barrier left":
          wObjectsInLevel[x][y] = new Ground(x, y, whichGround);
          wObjectsInLevel[x+1][y] = new Ground(x+1, y, ((DoorPiece) wObjectsInLevel[x+1][y]).whichGround);
          rect(x, y, 1, 1);
          rect(x+1, y, 1, 1);
          x = x+2;
          wObjectsInLevel[x][y] = this;
          break;
        case "barrier middle": 
          rect(x, y, 1, 1);
          x = x+2;
          wObjectsInLevel[x][y] = this;
          break;
        case "barrier right": 
          rect(x, y, 1, 1);
          x = x+2;
          wObjectsInLevel[x][y] = this;
          break;
          // door
        case "door top":
          wObjectsInLevel[x][y] = new Ground(x, y, whichGround);
          rect(x, y, 1, 1);
          x = x+2;
          y = y+2;
          whichObject = "door right";
          wObjectsInLevel[x][y] = this;
          break;
        case "door bottom":
          wObjectsInLevel[x][y] = new Ground(x, y, whichGround);
          rect(x, y, 1, 1);
          x = x+1;
          y = y+1;
          whichObject = "door left";
          wObjectsInLevel[x][y] = this;
          break;
        default:
          // do nothing
        }
      } 
      // from other state to original
      else {
        originalState = true;
        fill(grm.world.bg);                  // color for redrawing the background on changing tiles
        rectMode(CORNER);
        switch (whichObject) {
          // garden gate
        case "garden gate bottom":
          wObjectsInLevel[x][y] = new Ground(x, y, whichGround);
          rect(x, y, 1, 1);
          x = x+1;
          y = y+1;
          whichObject = "garden gate left";
          wObjectsInLevel[x][y] = this;
          break;
        case "garden gate top":
          wObjectsInLevel[x][y] = new Ground(x, y, whichGround);
          rect(x, y, 1, 1);
          x = x+2;
          y = y+2;
          whichObject = "garden gate right";
          wObjectsInLevel[x][y] = this;
          break;
          // barrier
        case "barrier left":
          rect(x, y, 1, 1);
          x = x-2;
          wObjectsInLevel[x][y] = this;
          break;
        case "barrier middle":
          rect(x, y, 1, 1);
          x = x-2;
          wObjectsInLevel[x][y] = this;
          break;
        case "barrier right":
          wObjectsInLevel[x][y] = new Ground(x, y, whichGround);
          wObjectsInLevel[x-1][y] = new Ground(x-1, y, ((DoorPiece) wObjectsInLevel[x-1][y]).whichGround);
          rect(x, y, 1, 1);
          rect(x-1, y, 1, 1);
          x = x-2;
          wObjectsInLevel[x][y] = this;
          break;
          // door
        case "door right":
          wObjectsInLevel[x][y] = new Ground(x, y, whichGround);
          rect(x, y, 1, 1);
          x = x-2;
          y = y-2;
          whichObject = "door top";
          wObjectsInLevel[x][y] = this;
          break;
        case "door left":
          wObjectsInLevel[x][y] = new Ground(x, y, whichGround);
          rect(x, y, 1, 1);
          x = x-1;
          y = y-1;
          whichObject = "door bottom";
          wObjectsInLevel[x][y] = this;
          break;
        default:
          // do nothing
        }
      }

      timeLastActed = millis();
      acted = true;

      // other pieces of the same door act too
      for (int i = 0; i < doorPieces.size(); i++) {
        if (doorPieces.get(i) != this && doorPieces.get(i).doorID == this.doorID) {
          doorPieces.get(i).act();
        }
      }
    }
  }
}
