import java.lang.Math.*;

boolean[] keys = new boolean[255];
int movmentTimerP1 = 0;
int movmentTimerP2 = 0;
int movmentThreshold = 1;
int timer;
int timeOutTime = 120;
int maxUfoPieces;

float lastMoved = 0;
boolean move = true;
final float MOVEMENT_COOLDOWN = 600;

class GameRunningMode extends GameMode {

  public World world;
  float deltaTime;
  float startTime = 0;
  public int counter = 0;
  PGraphics pg = createGraphics(100, 50);
  boolean allUfoPiecesCollected;
  float goalAlpha = 1;
  boolean goalAlphaDeclining = false;
  float goalAnimationSpeed = 30;
  PImage imgGoal = loadImage("levels/goal.png");
  PFont font = createFont("font/lithospro-regular.otf", 40);

  public GameRunningMode(Game game, ControlP5 cp5, UI ui) {
    super(game, cp5, ui);
    ModeName = "GameRunningMode";
  }

  void start(boolean fromLevel2) {
    ui.updateUI();

    movmentTimerP1 = movmentThreshold + 1;
    movmentTimerP2 = movmentThreshold + 1;

    timer = timeOutTime;
    startTime = millis() / 1000;
    counter = 0;
  }

  //Timer
  void time() {
    deltaTime =  (millis() / 1000) - startTime ;

    if (timer > 0 ) {
      float currentTimer = timeOutTime - deltaTime;
      timer = (int) currentTimer +1;
    } else {
      MainGame.currentGameMode = MainGame.glm;
      MainGame.currentGameMode.start(false);
    }

    //Timer with Background so it doesn't get overwritten by world
    pushMatrix();
    String _timer = " " + timer;
    pg.beginDraw();
    if (timer >= 20) {
      pg.background(world.bg);
    } else {
      pg.background(#FF0000);
    }
    pg.fill(245, 227, 154);
    pg.textFont(font);
    pg.textAlign(CENTER, CENTER);
    pg.text(_timer, 40, 20);
    pg.endDraw();
    image(pg, 0, 0, 4, 2);
    popMatrix();
    //---------------------------------------------------------
  }
  //---------------------------------------------------------


  void addUfoPiece() {
    counter += 1;
    if (counter >= maxUfoPieces) {
      allUfoPiecesCollected = true;
    }
  }

  void drawCounter() {
    pushMatrix();
    translate(0, 2);
    pg.beginDraw();
    pg.background(world.bg);
    pg.fill(245, 227, 154);
    pg.textFont(font);
    pg.textAlign(CENTER, CENTER);
    pg.text("C: " + counter, 50, 20);
    pg.endDraw();
    image(pg, 0, 0, 4, 2);
    popMatrix();
  }

  void showGoal() {
    fill(#148B06);
    rectMode(CORNER);
    rect(goalX1, goalY1, 1, 1);
    rect(goalX2, goalY2, 1, 1);
    if (goalAlpha >= 255) {
      goalAlphaDeclining = true;
    } else if (goalAlpha <= 10) {
      goalAlphaDeclining = false;
    }
    if (goalAlphaDeclining) {
      goalAlpha -= goalAnimationSpeed;
    } else {
      goalAlpha += goalAnimationSpeed;
    }
    tint(255, goalAlpha);
    image (imgGoal, goalX1, goalY1, 1, 1);
    image (imgGoal, goalX2, goalY2, 1, 1);
    noTint();
  }

  boolean aliensInGoal() {
    if (alien1.inGoal && alien2.inGoal) {
      return true;
    } else {
      return false;
    }
  }


  PVector alien1Input = new PVector(0, 0);
  PVector alien2Input = new PVector(0, 0);

  //Creates World, Player, KI
  void drawGame() {
    world.resetLitTiles();

    world.draw();

    //Condition that if Alines collected all UfoPieces they have to get to finsih line which will appear after condition is true
    if (allUfoPiecesCollected) {
      showGoal();
    }
    //---------------------------------------------------------

    // draw and move humans
    if (!move && millis()-lastMoved >= MOVEMENT_COOLDOWN) {
      move = true;
      lastMoved = millis();
    }
    for (Human human : humans) {
      if (move) {
        human.move();
      }
      human.draw();
    }
    move = false;
    //---------------------------------------------------------

    // alien movement and draw
    alien1Input.x = 0;
    alien1Input.y = 0;

    alien2Input.x = 0;
    alien2Input.y = 0;

    for (int i = 0; i < 255; i++) {
      //Array saves every key to detect if pressed key is in it and requiers action
      if (keys[i]) { 
        if (i == 87 ) { 
          alien1Input.y = - 1;
          movmentTimerP1 = 0;
        }
        if (i == 65) { 
          alien1Input.x = - 1;
          movmentTimerP1 = 0;
        }
        if (i == 83 ) { 
          alien1Input.y = 1;
          movmentTimerP1 = 0;
        }
        if (i == 68) { 
          alien1Input.x = 1;
          movmentTimerP1 = 0;
        }
        if (i == 38) { 
          alien2Input.y = -1;
          movmentTimerP2 = 0;
        }
        if (i == 40) { 
          alien2Input.y = 1;
          movmentTimerP2 = 0;
        }
        if (i == 37) { 
          alien2Input.x = -1;
          movmentTimerP2 = 0;
        }
        if (i == 39) {
          alien2Input.x = 1;
          movmentTimerP2 = 0;
        }
      }
    }
    //---------------------------------------------------------

    alien1Input.normalize();
    alien2Input.normalize();
    if (alien1Input.x != 0 || alien1Input.y != 0) {
      alien1.recieveInput((int) alien1Input.x, (int) alien1Input.y);
    }
    if (alien2Input.x != 0 || alien2Input.y != 0) {
      alien2.recieveInput((int) alien2Input.x, (int) alien2Input.y);
    }

    alien1.draw();
    alien2.draw();
    //---------------------------------------------------------

    //draw lights                                               
    for (int x = 0; x < NUM_TILES_X; x++) {
      for (int y = 0; y < NUM_TILES_Y; y++) {
        if (litTiles[x][y]) {
          rectMode(CORNER);
          fill(#FFFBD1, 130);
          rect(x, y, 1, 1);
        }
      }
    }
    //---------------------------------------------------------

    // check if win conditions are met
    if (allUfoPiecesCollected) {
      if (aliensInGoal()) {
        MainGame.currentGameMode = MainGame.gwm;
        if (world.level == 3) {
          MainGame.currentGameMode.start(true);
        } else {
          MainGame.currentGameMode.start(false);
        }
      }
    }
    //---------------------------------------------------------

    time();
    drawCounter();
  }
  //---------------------------------------------------------
}

public void keyPressed() {
  if (currentGameMode == gtm ) {
    startLevel(1);
  }
  if (keyCode >= keys.length || keyCode < 0) {
    return;
  }

  keys[keyCode] = true;

  if (keyCode == 87 || keyCode == 65 || keyCode == 83 || keyCode == 68) {
    movmentTimerP1++;
  } else {
    movmentTimerP1 = movmentThreshold + 1;
  }
  if (keyCode == 38 || keyCode == 40 || keyCode == 39 || keyCode == 37) {
    movmentTimerP2++;
  } else {
    movmentTimerP2 = movmentThreshold + 1;
  }
}

public void keyReleased() {
  if (keyCode >= keys.length || keyCode < 0) {
    return;
  }
  keys[keyCode] = false;

  if (keyCode == 87 || keyCode == 65 || keyCode == 83 || keyCode == 68) {
    movmentTimerP1 = movmentThreshold + 1;
  }
  if (keyCode == 38 || keyCode == 40 || keyCode == 39 || keyCode == 37) {
    movmentTimerP2 = movmentThreshold + 1;
  }
}
