class Lamp extends InteractiveWorldObject {
  boolean on;
  int radius = 3;
  boolean [][] litByLamp = new boolean [NUM_TILES_X][NUM_TILES_Y];
  int timeLastActed = timeOutTime;
  AudioPlayer lampNoise = minim.loadFile("sound/zapsplat_household_light_switch_garage_flick_001_53168.mp3");

  Lamp(int _x, int _y, String _whichObject) {
    x = _x;
    y = _y;
    walkable = false;
    whichObject = _whichObject;
    on = true;
  }
  //---------------------------------------------------------

  void draw() {
    imageMode(CORNER);
    image(lampImg, x, y, 1, 1);

    fov();
    lightTiles();
  }
  //---------------------------------------------------------

  /*
  fills the litByLamp array with tiles that are lit by the lamp. They are based on the radius and the surrounding geometry
  see fov() in Human class
   */
  public void fov() {
    // reset array
    for (int i= 0; i<NUM_TILES_X; i++) {                              
      for (int j = 0; j<NUM_TILES_Y; j++) {
        litByLamp[i][j] = false;
      }
    }

    float endOfLineX;
    float endOfLineY;
    final float DEG_RAD_CONV = PI/180; 

    if (on) {
      for (int i = 0; i < 360; i += 15) {
        float deg = i*DEG_RAD_CONV;

        endOfLineX = (cos(deg) * radius) + x;                      // lines to endpoints
        endOfLineY = (sin(deg) * radius) + y;

        int d = round(dist(x, y, endOfLineX, endOfLineY));

        for (int j = 0; j < d; j++) {
          int tileX = round(lerp(x, endOfLineX, j/(float)d));      // interpolate along the lines
          int tileY = round(lerp(y, endOfLineY, j/(float)d));        

          if (tileX < 0 || tileX >= NUM_TILES_X) {                 // check bounds
            continue;
          }    
          if (tileY < 0 || tileY >= NUM_TILES_Y) {
            continue;
          }

          if (!wObjectsInLevel[tileX][tileY].walkable) {           // if you hit a wall, door or other, dont continue
            // Lamp and NoiseItem are exceptions
            if (wObjectsInLevel[tileX][tileY] instanceof Lamp || wObjectsInLevel[tileX][tileY] instanceof NoiseItem) {
            } else {
              break;
            }
          }
          litByLamp[tileX][tileY] = true;
        }
      }
    }
  }
  //---------------------------------------------------------

  public void lightTiles() {
    for (int i= 0; i<NUM_TILES_X; i++) {                              
      for (int j = 0; j<NUM_TILES_Y; j++) {
        if (!litTiles[i][j]) {
          litTiles[i][j] = litByLamp[i][j];
        }
      }
    }
  }
  //---------------------------------------------------------

  void act() {
    if (timeLastActed > timer) {
      if (on) {
        on = false;
        for (int i= 0; i<NUM_TILES_X; i++) {                              
          for (int j = 0; j<NUM_TILES_Y; j++) {
            litByLamp[i][j] = false;
          }
        }
      } else {
        on = true;
      }
      timeLastActed = timer;

      this.lampNoise.rewind();
      this.lampNoise.play();
    }
  }
}
